/*
 *
 *  PDAYSUP.C --
 *
 *    POEE/Erisian/Discordian/Illuminati date routines.  All the preceding
 *    capitilized names have been shortened to POEE for uhh... some reason.
 *
 *    Now with extras!  Calculates astronomical stuffs such as the moon phase,
 *    epact, golden number, and Julian day number (JDN).
 *
 *  For them to make sense, the calendar conversions require one to read
 *  the Principia Discordia by Macalypse the Younger or maybe Illumantus!
 *  by Bob Shea and Robert Anton Wilson.
 *
 *
 *  Arguments used in most functions
 *    daynumber = Gregorian day of year (1-366)
 *    year      = Gregorian year
 *
 *
 *  To do
 *   - use day of year 0 for St. Tibb's Day/Heiligefliegendekindersheissetag
 *
 *
 *  Revision history
 *  97-10-26: first release
 *
 *
 */

typedef unsigned UINT;          /* EMX/GCC complains this is redefined in */
typedef unsigned long ULONG;    /* os2ext.h... so?  =) */

#if defined(EMX)
typedef int YEAR;               /* ? */
#else
typedef signed int YEAR;
#endif



/*
** Return: POEE day of year (1-366).  Simply subtract one if after the it's
** St. Tibb's Day.
**
** Phhft: DAYNUM.C seems to already subtract one day; if yours doesn't, this
** would serve some purpose.
**
*/

UINT poeedayyear(UINT daynumber)
{

#ifdef NOTUSED /* programming a la crack */
 if (isleap(year))
  return (daynumber> 60 ? daynumber-1 : daynumber);  /* ? */
  else return (daynumber);

#else
  return daynumber;

#endif

 }


/*
** Return: POEE day of week (0-5, 0=Sweetmorn)
**
*/

UINT poeedayweek(UINT daynumber)
{
 unsigned int joe;

 joe=poeedayyear(daynumber) % 5L;
 return (joe ? joe : 5 );

 }


/*
** Return: POEE day of season (1-73).
**
*/

UINT poeedayseason(UINT daynumber, YEAR year)
{
 unsigned int joe, sue;

 joe = poeedayyear(daynumber);
 if (daynumber>60 && isleap(year))
  sue = (joe-1) % 73L;
  else sue = joe % 73L;

 if (sue==0)
  return 73;
  else return sue;

 }


/*
** Return: POEE calendar season (0-4, 0=Chaos).
**
*/

UINT poeeseason(UINT daynumber, YEAR year)
{

 return ((poeedayyear(daynumber)/73) - (poeedayseason(daynumber,year)==73 ? 1 : 0));

 }


/*
** Return: year's Golden Number (1-19).
** Warning: untested.
**
*/

#define goldennumber(year)      ((year%19)+1)


/*
** Return:  year's Epact (1-30), Gregorian method.
** Accepts: positive years.
** Warning: untested.
**
** "In the Gregorian calendar, the Epact is the age of the moon at the
** start of the year."
**
** Adapted to C from the Calendar FAQ.
**
*/

UINT epact(YEAR year)
{
 UINT epact=0, century;

 century=(year/100)+1;

 epact = (11 * (goldennumber(year)-1)) % 30;
 epact = epact - (3*century)/4;
 epact = epact + (8*century + 5)/25;
 epact = epact + 8;
/* 5) Add or subtract 30 until the Epact lies between 1 and 30. */
 if (epact>30)
  epact-=30;

 return epact;

 }


/*
** Return: calculated indiction.
** Warning: untested.
**
*/

#define indiction(year)   ((year + 2) % 15 + 1)


/*
** Return: Julian day number (JDN), Gregorian method.
** Warning: gives abridged JDN or just doesn't work
**
** "The algorithm works fine for AD dates. If you want to use it for BC
** dates, you must first convert the BC year to a negative year (e.g,
** 10 BC = -9) and make sure that your integer divisions round down, not
** towards zero: -9/4 should be -3, not -2."
**
** Adapted to C from the Calendar FAQ.
**
*/

JDN(UINT year, UINT month, UINT day)
{
 int a, y, m, JDN;

 a = (14-month)/12;
 y = year-a;
 m = month + 12*a - 3;
 JDN = day + (306*m+5)/10 + y*365 + y/4 - y/100 + y/400 + 1721119;

 return JDN;
 }


/*
** Return: phase of the moon for a certian point in time (0-7, 0=new).
**
** Stolen from Doorskel, where it was stolel from Lynn Nash, who hacked it
** from BSD Unix, I think.
**
*/

UINT Moon_Phase(int year, int diy)

{
  int             phase;        /* Moon phase                  */
  int             cent;         /* Century number (1979 = 20)  */
  int             epact;        /* Age of the moon on Jan. 1   */
/*  int             diy;           Day in the year             */
  int             golden;       /* Moon's golden number        */
  char            yrstr[32],
		  was;
  int             c;



  cent = (year / 100) + 1;      /* Century number       */
  golden = (year % 19) + 1;     /* Golden number        */
  epact = ((11 * golden) + 20   /* Golden number        */
	   + (((8 * cent) + 5) / 25) - 5  /* 400 year cycle       */
	   - (((3 * cent) / 4) - 12)) % 30; /* Leap year correction */
  if (epact <= 0)
    epact += 30;                /* Age range is 1 .. 30 */
  if ((epact == 25 && golden > 11) || epact == 24)
    epact++;
  /* Calculate the phase, using the magic numbers defined above. Note that
   * (phase and 7) is equivalent to (phase mod 8) and is needed on two days
   * per year (when the algorithm yields 8). */
  phase = (((((diy + epact) * 6) + 11) % 177) / 22) & 7;

  return phase;
}


/*
** Return: is date valid in the Gregorian calendar?  (0 = false, 1 = true).
** From: DAYNUM.C by Bob Stout.
**
** Modified to not accept the year '0' and slightly optimized.
**
*/

Boolean_T valiDate_(unsigned yr, unsigned mo, unsigned day)
{
      unsigned int days[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

      if ( (1 > mo || 12 < mo) ||
	   (1 > day || day > (days[mo - 1] + (2 == mo && isleap(yr)))) ||
	   (yr==0) )
	    return False_;
      else  return True_;

}


#undef UINT
#undef ULONG

/* End of file "PDAYSUP.C" */

